from .enums import *
from .diagnostics import Diagnostic
import re


class QueryParser:
    """
    This class contains commonly used methods for initial parsing of a GET
    query. It does not include platform-specific methods.
    """

    rxTermQuery = re.compile('^(?:(?:[^ "]|\\\\")*|"(?:[^"]|\\\\")*")$')

    def __init__(self):
        pass

    @staticmethod
    def find_operator(strQuery, start=0, end=-1):
        if end == -1:
            end = len(strQuery) - 1
        if strQuery[start:start+3] == 'NOT':
            return start, 'NOT'
        parenthBalance = 0
        inQuotes = False
        for i in range(start, end):
            if inQuotes:
                if strQuery[i] == '"':
                    inQuotes = False
                continue
            if strQuery[i] == '"':
                inQuotes = True
                continue
            if strQuery[i] == '(':
                parenthBalance += 1
            elif strQuery[i] == ')':
                parenthBalance -= 1
            elif parenthBalance == 0:
                if strQuery[i:i+3] == 'AND':
                    return i, 'AND'
                elif strQuery[i:i+2] == 'OR':
                    return i, 'OR'
        return -1, ''

    def validate_query(self, operation, version, queryType, query,
                       xFcsEndpointDescription, xFcsContext,
                       xFcsDataviews, xFcsRewritesAllowed):
        """
        Check if the query parameters contain a valid combination of values.
        :param operation:
        :param version:
        :param queryType:
        :param query:
        :param xFcsEndpointDescription:
        :param xFcsContext:
        :param xFcsDataviews:
        :param xFcsRewritesAllowed:
        :return: Return a list of diagnostics describing problems with the query.
        If the query is prima facie valid and can be processed further, an empty
        list will be returned.
        """
        diagnostics = []

        # Check if additional parameters combine with the operation requested
        # (FCS specifications, 4.1)
        if len(xFcsEndpointDescription) > 0 and operation != Operation.explain:
            diagnostics.append(Diagnostic(DiagnosticTypes.sru, 8, details='x-fcs-endpoint-description'))
        if len(xFcsContext) > 0 and operation != Operation.searchRetrieve:
            diagnostics.append(Diagnostic(DiagnosticTypes.sru, 8, details='x-fcs-context'))
        if len(xFcsDataviews) > 0 and operation != Operation.searchRetrieve:
            diagnostics.append(Diagnostic(DiagnosticTypes.sru, 8, details='x-fcs-dataviews'))
        if len(xFcsRewritesAllowed) > 0 and operation != Operation.searchRetrieve:
            diagnostics.append(Diagnostic(DiagnosticTypes.sru, 8, details='x-fcs-rewrites-allowed'))

        return diagnostics


if __name__ == '__main__':
    pass

